@@ -1,16 +1,20 @@
+# -*-: coding utf-8 -*-
 """ This module contains the error-related constants and classes. """
 
 from __future__ import absolute_import
 
 from collections import defaultdict, namedtuple, MutableMapping
 from copy import copy, deepcopy
+from functools import wraps
 from pprint import pformat
 
+from cerberus.platform import PYTHON_VERSION
 from cerberus.utils import compare_paths_lt, quote_string
 
 
+ErrorDefinition = namedtuple('cerberus_error', 'code, rule')
 """
-Error definition constants
+Error definition class
 
 Each distinguishable error is defined as a two-value-tuple that holds
 a *unique* error id as integer and the rule as string that can cause it.
@@ -19,7 +23,6 @@ The names do not contain a common prefix as they are supposed to be referenced
 within the module namespace, e.g. errors.CUSTOM
 """
 
-ErrorDefinition = namedtuple('cerberus_error', 'code, rule')
 
 # custom
 CUSTOM = ErrorDefinition(0x00, None)
@@ -86,7 +89,7 @@ SCHEMA_ERROR_MISSING = "validation schema missing"
 """ Error representations """
 
 
-class ValidationError:
+class ValidationError(object):
     """ A simple class to store and query basic error information. """
     def __init__(self, document_path, schema_path, code, rule, constraint,
                  value, info):
@@ -316,7 +319,7 @@ class SchemaErrorTree(ErrorTree):
     tree_type = 'schema'
 
 
-class BaseErrorHandler:
+class BaseErrorHandler(object):
     """ Base class for all error handlers.
         Subclasses are identified as error-handlers with an instance-test. """
     def __init__(self, *args, **kwargs):
@@ -388,6 +391,33 @@ class ToyErrorHandler(BaseErrorHandler):
         pass
 
 
+def encode_unicode(f):
+    """Cerberus error messages expect regular binary strings.
+    If unicode is used in a ValidationError message can't be printed.
+
+    This decorator ensures that if legacy Python is used unicode
+    strings are encoded before passing to a function.
+    """
+    @wraps(f)
+    def wrapped(obj, error):
+
+        def _encode(value):
+            """Helper encoding unicode strings into binary utf-8"""
+            if isinstance(value, unicode):  # noqa: F821
+                return value.encode('utf-8')
+            return value
+
+        error = copy(error)
+        error.document_path = _encode(error.document_path)
+        error.schema_path = _encode(error.schema_path)
+        error.constraint = _encode(error.constraint)
+        error.value = _encode(error.value)
+        error.info = _encode(error.info)
+        return f(obj, error)
+
+    return wrapped if PYTHON_VERSION < 3 else f
+
+
 class BasicErrorHandler(BaseErrorHandler):
     """ Models cerberus' legacy. Returns a :class:`dict`. """
     messages = {0x00: "{0}",
@@ -446,6 +476,7 @@ class BasicErrorHandler(BaseErrorHandler):
     def __str__(self):
         return pformat(self.pretty_tree)
 
+    @encode_unicode
     def add(self, error):
         if error.is_logic_error:
             self.insert_logic_error(error)
@@ -459,9 +490,9 @@ class BasicErrorHandler(BaseErrorHandler):
         self.tree = {}
 
     def format_message(self, field, error):
-        return self.messages[error.code]\
-            .format(*error.info, constraint=error.constraint,
-                    field=field, value=error.value)
+        return self.messages[error.code].format(
+            *error.info, constraint=error.constraint,
+            field=field, value=error.value)
 
     def insert_error(self, path, node):
         """ Adds an error or sub-tree to :attr:tree.
@@ -513,9 +544,9 @@ class BasicErrorHandler(BaseErrorHandler):
             nodename = '%s definition %s' % (error.rule, i)
             for child_error in child_errors:
                 if child_error.is_logic_error:
-                    raise NotImplemented
+                    raise NotImplementedError
                 elif child_error.is_group_error:
-                    raise NotImplemented
+                    raise NotImplementedError
                 else:
                     self.insert_error(path + (nodename,),
                                       self.format_message(field, child_error))
