@@ -169,7 +169,8 @@ class Validator(object):
                 kwargs[p] = args[i]
         self._config = kwargs
         """ This dictionary holds the configuration arguments that were used to
-            initialize the :class:`Validator` instance. """
+            initialize the :class:`Validator` instance except the
+            ``error_handler``. """
 
     @classmethod
     def clear_caches(cls):
@@ -322,6 +323,36 @@ class Validator(object):
                 self._drop_nodes_from_errorpaths(error.child_errors,
                                                  dp_items, sp_items)
 
+    def _lookup_field(self, path):
+        """ Searches for a field as defined by path. This method is used by the
+            ``dependency`` evaluation logic.
+
+        :param path: Path elements are separated by a ``.``. A leading ``^``
+                     indicates that the path relates to the document root,
+                     otherwise it relates to the currently evaluated document,
+                     which is possibly a subdocument.
+                     The sequence ``^^`` at the start will be interpreted as a
+                     literal ``^``.
+        :type path: :class:`str`
+        :returns: Either the found field name and its value or :obj:`None` for
+                  both.
+        :rtype: A two-value :class:`tuple`.
+        """
+        if path.startswith('^'):
+            path = path[1:]
+            context = self.document if path.startswith('^') \
+                else self.root_document
+        else:
+            context = self.document
+
+        parts = path.split('.')
+        for part in parts:
+            context = context.get(part)
+            if context is None:
+                return None, None
+
+        return parts[-1], context
+
     def _resolve_rules_set(self, rules_set):
         if isinstance(rules_set, Mapping):
             return rules_set
@@ -376,6 +407,15 @@ class Validator(object):
         Type: :class:`bool` """
         return self._config.get('is_child', False)
 
+    @property
+    def is_normalized(self):
+        """ ``True`` if the document is already normalized. """
+        return self._config.get('is_normalized', False)
+
+    @is_normalized.setter
+    def is_normalized(self, value):
+        self._config['is_normalized'] = value
+
     @property
     def purge_unknown(self):
         """ If ``True`` unknown fields will be deleted from the document
@@ -491,9 +531,19 @@ class Validator(object):
             return self.document
 
     def __normalize_mapping(self, mapping, schema):
+        if isinstance(schema, _str_type):
+            schema = self._resolve_schema(schema)
+        schema = schema.copy()
+        for field in schema:
+            schema[field] = self._resolve_rules_set(schema[field])
+
         self.__normalize_rename_fields(mapping, schema)
         if self.purge_unknown:
             self._normalize_purge_unknown(mapping, schema)
+        # Check `readonly` fields before applying default values because
+        # a field's schema definition might contain both `readonly` and
+        # `default`.
+        self.__validate_readonly_fields(mapping, schema)
         self.__normalize_default_fields(mapping, schema)
         self._normalize_coerce(mapping, schema)
         self.__normalize_containers(mapping, schema)
@@ -600,7 +650,7 @@ class Validator(object):
     def __normalize_mapping_per_schema(self, field, mapping, schema):
         validator = self._get_child_validator(
             document_crumb=field, schema_crumb=(field, 'schema'),
-            schema=self._resolve_schema(schema[field].get('schema', {})),
+            schema=schema[field]['schema'],
             allow_unknown=schema[field].get('allow_unknown', self.allow_unknown),  # noqa
             purge_unknown=schema[field].get('purge_unknown', self.purge_unknown))  # noqa
         mapping[field] = validator.normalized(mapping[field],
@@ -610,7 +660,7 @@ class Validator(object):
 
     def __normalize_sequence(self, field, mapping, schema):
         schema = dict(((k, schema[field]['schema'])
-                      for k in range(len(mapping[field]))))
+                       for k in range(len(mapping[field]))))
         document = dict((k, v) for k, v in enumerate(mapping[field]))
         validator = self._get_child_validator(
             document_crumb=field, schema_crumb=(field, 'schema'),
@@ -664,6 +714,12 @@ class Validator(object):
             mapping[new_name] = mapping[field]
             del mapping[field]
 
+    def __validate_readonly_fields(self, mapping, schema):
+        for field in (x for x in schema if x in mapping and
+                      self._resolve_rules_set(schema[x]).get('readonly')):
+            self._validate_readonly(schema[field]['readonly'], field,
+                                    mapping[field])
+
     def __normalize_default_fields(self, mapping, schema):
         fields = [x for x in schema if x not in mapping or
                   mapping[x] is None and not schema[x].get('nullable', False)]
@@ -737,6 +793,7 @@ class Validator(object):
         self.__init_processing(document, schema)
         if normalize:
             self.__normalize_mapping(self.document, self.schema)
+            self.is_normalized = True
 
         for field in self.document:
             if self.ignore_none_values and self.document[field] is None:
@@ -757,8 +814,9 @@ class Validator(object):
     __call__ = validate
 
     def validated(self, *args, **kwargs):
-        """ Wrapper around :func:`validate` that returns the normalized and
-            validated document or :obj:`None` if validation failed. """
+        """ Wrapper around :meth:`~cerberus.Validator.validate` that returns
+            the normalized and validated document or :obj:`None` if validation
+            failed. """
         always_return_document = kwargs.pop('always_return_document', False)
         self.validate(*args, **kwargs)
         if self._errors and not always_return_document:
@@ -844,41 +902,31 @@ class Validator(object):
             return True
 
     def __validate_dependencies_mapping(self, dependencies, field):
-        validated_deps = 0
-        for dep_name, dep_values in dependencies.items():
-            if (not isinstance(dep_values, Sequence) or
-                    isinstance(dep_values, _str_type)):
-                dep_values = [dep_values]
-            context = self.document
-            parts = dep_name.split('.')
-            info = {}
+        validated_dependencies_counter = 0
+        error_info = {}
+        for dependency_name, dependency_values in dependencies.items():
+            if (not isinstance(dependency_values, Sequence) or
+                    isinstance(dependency_values, _str_type)):
+                dependency_values = [dependency_values]
 
-            for part in parts:
-                if part in context:
-                    context = context[part]
-                    if context in dep_values:
-                        validated_deps += 1
-                    else:
-                        info.update({dep_name: context})
+            wanted_field, wanted_field_value = \
+                self._lookup_field(dependency_name)
+            if wanted_field_value in dependency_values:
+                validated_dependencies_counter += 1
+            else:
+                error_info.update({dependency_name: wanted_field_value})
 
-        if validated_deps != len(dependencies):
-            self._error(field, errors.DEPENDENCIES_FIELD_VALUE, info)
+        if validated_dependencies_counter != len(dependencies):
+            self._error(field, errors.DEPENDENCIES_FIELD_VALUE, error_info)
 
     def __validate_dependencies_sequence(self, dependencies, field):
         for dependency in dependencies:
-
-            context = self.document.copy()
-            parts = dependency.split('.')
-
-            for part in parts:
-                if part in context:
-                    context = context[part]
-                else:
-                    self._error(field, errors.DEPENDENCIES_FIELD, dependency)
+            if self._lookup_field(dependency)[0] is None:
+                self._error(field, errors.DEPENDENCIES_FIELD, dependency)
 
     def _validate_empty(self, empty, field, value):
         """ {'type': 'boolean'} """
-        if isinstance(value, _str_type) and len(value) == 0 and not empty:
+        if isinstance(value, Iterable) and len(value) == 0 and not empty:
             self._error(field, errors.EMPTY_NOT_ALLOWED)
 
     def _validate_excludes(self, excludes, field, value):
@@ -891,7 +939,7 @@ class Validator(object):
             self._unrequired_by_excludes.add(field)
         for exclude in excludes:
             if (exclude in self.schema and
-               'required' in self.schema[exclude] and
+                'required' in self.schema[exclude] and
                     self.schema[exclude]['required']):
 
                 self._unrequired_by_excludes.add(exclude)
@@ -1004,12 +1052,12 @@ class Validator(object):
     def _validate_maxlength(self, max_length, field, value):
         """ {'type': 'integer'} """
         if isinstance(value, Iterable) and len(value) > max_length:
-                self._error(field, errors.MAX_LENGTH, len(value))
+            self._error(field, errors.MAX_LENGTH, len(value))
 
     def _validate_minlength(self, min_length, field, value):
         """ {'type': 'integer'} """
         if isinstance(value, Iterable) and len(value) < min_length:
-                self._error(field, errors.MIN_LENGTH, len(value))
+            self._error(field, errors.MIN_LENGTH, len(value))
 
     def _validate_nullable(self, nullable, field, value):
         """ {'type': 'boolean'} """
@@ -1037,8 +1085,17 @@ class Validator(object):
     def _validate_readonly(self, readonly, field, value):
         """ {'type': 'boolean'} """
         if readonly:
-            self._error(field, errors.READONLY_FIELD)
-            return True
+            if not self.is_normalized:
+                self._error(field, errors.READONLY_FIELD)
+                return True
+            # If the document was normalized (and therefore already been
+            # checked for readonly fields), we still have to return True
+            # if an error was filed.
+            has_error = errors.READONLY_FIELD in \
+                self.document_error_tree.fetch_errors_from(
+                    self.document_path + (field,))
+            if self.is_normalized and has_error:
+                return True
 
     def _validate_regex(self, pattern, field, value):
         """ {'type': 'string'} """
@@ -1053,9 +1110,7 @@ class Validator(object):
     _validate_required = dummy_for_rule_validation(""" {'type': 'boolean'} """)
 
     def __validate_required_fields(self, document):
-        """ Validates that required fields are not missing. If dependencies
-            are precised then validate 'required' only if all dependencies
-            are validated.
+        """ Validates that required fields are not missing.
 
         :param document: The document being validated.
         """
@@ -1211,7 +1266,7 @@ class Validator(object):
                 self._error(field, errors.VALUESCHEMA, validator._errors)
 
 
-RULE_SCHEMA_SEPERATOR = \
+RULE_SCHEMA_SEPARATOR = \
     "The rule's arguments are validated against this schema:"
 
 
@@ -1229,17 +1284,16 @@ class InspectedValidator(type):
 
         super(InspectedValidator, cls).__init__(*args)
 
-        cls.types, cls.validators, cls.validation_rules = (), (), {}
+        cls.types, cls.validation_rules = (), {}
         for attribute in attributes_with_prefix('validate'):
             if attribute.startswith('type_'):
                 cls.types += (attribute[len('type_'):],)
-            elif attribute.startswith('validator_'):
-                cls.validators += (attribute[len('validator_'):],)
             else:
                 cls.validation_rules[attribute] = \
                     cls.__get_rule_schema('_validate_' + attribute)
-
         cls.validation_rules['type']['allowed'] = cls.types
+
+        cls.validators = tuple(x for x in attributes_with_prefix('validator'))
         x = cls.validation_rules['validator']['oneof']
         x[1]['schema']['oneof'][1]['allowed'] = x[2]['allowed'] = cls.validators
 
@@ -1272,8 +1326,8 @@ class InspectedValidator(type):
         if docstring is None:
             result = {}
         else:
-            if RULE_SCHEMA_SEPERATOR in docstring:
-                docstring = docstring.split(RULE_SCHEMA_SEPERATOR)[1]
+            if RULE_SCHEMA_SEPARATOR in docstring:
+                docstring = docstring.split(RULE_SCHEMA_SEPARATOR)[1]
             try:
                 result = literal_eval(docstring.strip())
             except Exception:
